@@ -3,10 +3,12 @@
 
 #include "config.h"
 
+#include <libavcodec/version.h>
 #include <libavformat/version.h>
+#include <libavutil/version.h>
 #include <SDL2/SDL_version.h>
 
-#ifndef __WIN32
+#ifndef _WIN32
 # define PRIu64_ PRIu64
 # define SC_PRIsizet "zu"
 #else
@@ -50,6 +52,15 @@
 # define SCRCPY_LAVU_HAS_CHLAYOUT
 #endif
 
+// In ffmpeg/doc/APIchanges:
+// 2023-10-06 - 5432d2aacad - lavc 60.15.100 - avformat.h
+//   Deprecate AVFormatContext.{nb_,}side_data, av_stream_add_side_data(),
+//   av_stream_new_side_data(), and av_stream_get_side_data(). Side data fields
+//   from AVFormatContext.codecpar should be used from now on.
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(60, 15, 100)
+# define SCRCPY_LAVC_HAS_CODECPAR_CODEC_SIDEDATA
+#endif
+
 #if SDL_VERSION_ATLEAST(2, 0, 6)
 // <https://github.com/libsdl-org/SDL/commit/d7a318de563125e5bb465b1000d6bc9576fbc6fc>
 # define SCRCPY_SDL_HAS_HINT_TOUCH_MOUSE_EVENTS
