@@ -9,6 +9,7 @@
 
 #include "config.h"
 #include "compat.h"
+#include "util/env.h"
 #include "util/file.h"
 #include "util/log.h"
 #include "util/str.h"
@@ -19,35 +20,22 @@
 
 static char *
 get_icon_path(void) {
-#ifdef __WINDOWS__
-    const wchar_t *icon_path_env = _wgetenv(L"SCRCPY_ICON_PATH");
-#else
-    const char *icon_path_env = getenv("SCRCPY_ICON_PATH");
-#endif
-    if (icon_path_env) {
+    char *icon_path = sc_get_env("SCRCPY_ICON_PATH");
+    if (icon_path) {
         // if the envvar is set, use it
-#ifdef __WINDOWS__
-        char *icon_path = sc_str_from_wchars(icon_path_env);
-#else
-        char *icon_path = strdup(icon_path_env);
-#endif
-        if (!icon_path) {
-            LOG_OOM();
-            return NULL;
-        }
         LOGD("Using SCRCPY_ICON_PATH: %s", icon_path);
         return icon_path;
     }
 
 #ifndef PORTABLE
     LOGD("Using icon: " SCRCPY_DEFAULT_ICON_PATH);
-    char *icon_path = strdup(SCRCPY_DEFAULT_ICON_PATH);
+    icon_path = strdup(SCRCPY_DEFAULT_ICON_PATH);
     if (!icon_path) {
         LOG_OOM();
         return NULL;
     }
 #else
-    char *icon_path = sc_file_get_local_path(SCRCPY_PORTABLE_ICON_FILENAME);
+    icon_path = sc_file_get_local_path(SCRCPY_PORTABLE_ICON_FILENAME);
     if (!icon_path) {
         LOGE("Could not get icon path");
         return NULL;
@@ -78,7 +66,19 @@ decode_image(const char *path) {
         goto close_input;
     }
 
-    int stream = av_find_best_stream(ctx, AVMEDIA_TYPE_VIDEO, -1, -1, NULL, 0);
+
+// In ffmpeg/doc/APIchanges:
+// 2021-04-27 - 46dac8cf3d - lavf 59.0.100 - avformat.h
+//   av_find_best_stream now uses a const AVCodec ** parameter
+//   for the returned decoder.
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(59, 0, 100)
+    const AVCodec *codec;
+#else
+    AVCodec *codec;
+#endif
+
+    int stream =
+        av_find_best_stream(ctx, AVMEDIA_TYPE_VIDEO, -1, -1, &codec, 0);
     if (stream < 0 ) {
         LOGE("Could not find best image stream");
         goto close_input;
@@ -86,12 +86,6 @@ decode_image(const char *path) {
 
     AVCodecParameters *params = ctx->streams[stream]->codecpar;
 
-    const AVCodec *codec = avcodec_find_decoder(params->codec_id);
-    if (!codec) {
-        LOGE("Could not find image decoder");
-        goto close_input;
-    }
-
     AVCodecContext *codec_ctx = avcodec_alloc_context3(codec);
     if (!codec_ctx) {
         LOG_OOM();
@@ -111,21 +105,21 @@ decode_image(const char *path) {
     AVFrame *frame = av_frame_alloc();
     if (!frame) {
         LOG_OOM();
-        goto close_codec;
+        goto free_codec_ctx;
     }
 
     AVPacket *packet = av_packet_alloc();
     if (!packet) {
         LOG_OOM();
         av_frame_free(&frame);
-        goto close_codec;
+        goto free_codec_ctx;
     }
 
     if (av_read_frame(ctx, packet) < 0) {
         LOGE("Could not read frame");
         av_packet_free(&packet);
         av_frame_free(&frame);
-        goto close_codec;
+        goto free_codec_ctx;
     }
 
     int ret;
@@ -133,22 +127,20 @@ decode_image(const char *path) {
         LOGE("Could not send icon packet: %d", ret);
         av_packet_free(&packet);
         av_frame_free(&frame);
-        goto close_codec;
+        goto free_codec_ctx;
     }
 
     if ((ret = avcodec_receive_frame(codec_ctx, frame)) != 0) {
         LOGE("Could not receive icon frame: %d", ret);
         av_packet_free(&packet);
         av_frame_free(&frame);
-        goto close_codec;
+        goto free_codec_ctx;
     }
 
     av_packet_free(&packet);
 
     result = frame;
 
-close_codec:
-    avcodec_close(codec_ctx);
 free_codec_ctx:
     avcodec_free_context(&codec_ctx);
 close_input:
