@@ -11,6 +11,7 @@
 #include "file_pusher.h"
 #include "fps_counter.h"
 #include "options.h"
+#include "trait/gamepad_processor.h"
 #include "trait/key_processor.h"
 #include "trait/mouse_processor.h"
 
@@ -21,6 +22,7 @@ struct sc_input_manager {
 
     struct sc_key_processor *kp;
     struct sc_mouse_processor *mp;
+    struct sc_gamepad_processor *gp;
 
     struct sc_mouse_bindings mouse_bindings;
     bool legacy_paste;
@@ -50,6 +52,7 @@ struct sc_input_manager_params {
     struct sc_screen *screen;
     struct sc_key_processor *kp;
     struct sc_mouse_processor *mp;
+    struct sc_gamepad_processor *gp;
 
     struct sc_mouse_bindings mouse_bindings;
     bool legacy_paste;
