@@ -13,6 +13,7 @@
 #include "fps_counter.h"
 #include "frame_buffer.h"
 #include "input_manager.h"
+#include "mouse_capture.h"
 #include "opengl.h"
 #include "options.h"
 #include "trait/key_processor.h"
@@ -30,6 +31,7 @@ struct sc_screen {
 
     struct sc_display display;
     struct sc_input_manager im;
+    struct sc_mouse_capture mc; // only used in mouse relative mode
     struct sc_frame_buffer fb;
     struct sc_fps_counter fps_counter;
 
@@ -61,10 +63,6 @@ struct sc_screen {
     bool maximized;
     bool minimized;
 
-    // To enable/disable mouse capture, a mouse capture key (LALT, LGUI or
-    // RGUI) must be pressed. This variable tracks the pressed capture key.
-    SDL_Keycode mouse_capture_key_pressed;
-
     AVFrame *frame;
 
     bool paused;
@@ -78,6 +76,7 @@ struct sc_screen_params {
     struct sc_file_pusher *fp;
     struct sc_key_processor *kp;
     struct sc_mouse_processor *mp;
+    struct sc_gamepad_processor *gp;
 
     struct sc_mouse_bindings mouse_bindings;
     bool legacy_paste;
@@ -125,9 +124,9 @@ sc_screen_destroy(struct sc_screen *screen);
 void
 sc_screen_hide_window(struct sc_screen *screen);
 
-// switch the fullscreen mode
+// toggle the fullscreen mode
 void
-sc_screen_switch_fullscreen(struct sc_screen *screen);
+sc_screen_toggle_fullscreen(struct sc_screen *screen);
 
 // resize window to optimal size (remove black borders)
 void
