@@ -8,19 +8,17 @@
 
 #include "coords.h"
 #include "trait/frame_sink.h"
-#include "video_buffer.h"
+#include "frame_buffer.h"
 #include "util/tick.h"
 
 struct sc_v4l2_sink {
     struct sc_frame_sink frame_sink; // frame sink trait
 
-    struct sc_video_buffer vb;
+    struct sc_frame_buffer fb;
     AVFormatContext *format_ctx;
     AVCodecContext *encoder_ctx;
 
     char *device_name;
-    struct sc_size frame_size;
-    sc_tick buffering_time;
 
     sc_thread thread;
     sc_mutex mutex;
@@ -34,8 +32,7 @@ struct sc_v4l2_sink {
 };
 
 bool
-sc_v4l2_sink_init(struct sc_v4l2_sink *vs, const char *device_name,
-                  struct sc_size frame_size, sc_tick buffering_time);
+sc_v4l2_sink_init(struct sc_v4l2_sink *vs, const char *device_name);
 
 void
 sc_v4l2_sink_destroy(struct sc_v4l2_sink *vs);
