@@ -289,11 +289,11 @@ static void test_serialize_set_clipboard_long(void) {
     assert(!memcmp(buf, expected, sizeof(expected)));
 }
 
-static void test_serialize_set_screen_power_mode(void) {
+static void test_serialize_set_display_power(void) {
     struct sc_control_msg msg = {
-        .type = SC_CONTROL_MSG_TYPE_SET_SCREEN_POWER_MODE,
-        .set_screen_power_mode = {
-            .mode = SC_SCREEN_POWER_MODE_NORMAL,
+        .type = SC_CONTROL_MSG_TYPE_SET_DISPLAY_POWER,
+        .set_display_power = {
+            .on = true,
         },
     };
 
@@ -302,8 +302,8 @@ static void test_serialize_set_screen_power_mode(void) {
     assert(size == 2);
 
     const uint8_t expected[] = {
-        SC_CONTROL_MSG_TYPE_SET_SCREEN_POWER_MODE,
-        0x02, // SC_SCREEN_POWER_MODE_NORMAL
+        SC_CONTROL_MSG_TYPE_SET_DISPLAY_POWER,
+        0x01, // true
     };
     assert(!memcmp(buf, expected, sizeof(expected)));
 }
@@ -329,6 +329,8 @@ static void test_serialize_uhid_create(void) {
         .type = SC_CONTROL_MSG_TYPE_UHID_CREATE,
         .uhid_create = {
             .id = 42,
+            .vendor_id = 0x1234,
+            .product_id = 0x5678,
             .name = "ABC",
             .report_desc_size = sizeof(report_desc),
             .report_desc = report_desc,
@@ -337,11 +339,13 @@ static void test_serialize_uhid_create(void) {
 
     uint8_t buf[SC_CONTROL_MSG_MAX_SIZE];
     size_t size = sc_control_msg_serialize(&msg, buf);
-    assert(size == 20);
+    assert(size == 24);
 
     const uint8_t expected[] = {
         SC_CONTROL_MSG_TYPE_UHID_CREATE,
         0, 42, // id
+        0x12, 0x34, // vendor id
+        0x56, 0x78, // product id
         3, // name size
         65, 66, 67, // "ABC"
         0, 11, // report desc size
@@ -407,6 +411,21 @@ static void test_serialize_open_hard_keyboard(void) {
     assert(!memcmp(buf, expected, sizeof(expected)));
 }
 
+static void test_serialize_reset_video(void) {
+    struct sc_control_msg msg = {
+        .type = SC_CONTROL_MSG_TYPE_RESET_VIDEO,
+    };
+
+    uint8_t buf[SC_CONTROL_MSG_MAX_SIZE];
+    size_t size = sc_control_msg_serialize(&msg, buf);
+    assert(size == 1);
+
+    const uint8_t expected[] = {
+        SC_CONTROL_MSG_TYPE_RESET_VIDEO,
+    };
+    assert(!memcmp(buf, expected, sizeof(expected)));
+}
+
 int main(int argc, char *argv[]) {
     (void) argc;
     (void) argv;
@@ -423,11 +442,12 @@ int main(int argc, char *argv[]) {
     test_serialize_get_clipboard();
     test_serialize_set_clipboard();
     test_serialize_set_clipboard_long();
-    test_serialize_set_screen_power_mode();
+    test_serialize_set_display_power();
     test_serialize_rotate_device();
     test_serialize_uhid_create();
     test_serialize_uhid_input();
     test_serialize_uhid_destroy();
     test_serialize_open_hard_keyboard();
+    test_serialize_reset_video();
     return 0;
 }
